package http

import (
	"dwelling-radio/internal/radio"
	"dwelling-radio/pkg/oggtag"
	"encoding/json"
	"fmt"
	"log"
	"net/http"
	"strings"
	"time"
)

type DJHandlers struct {
	listeners    *radio.ListenerCounter
	playlist     *radio.Playlist
	songList     *radio.SongList
	mostLSong    *radio.MostListenedSong
	fallbackSong string
}

func NewDJHandlers(l *radio.ListenerCounter, p *radio.Playlist,
	sl *radio.SongList, mls *radio.MostListenedSong, fS string) *DJHandlers {
	return &DJHandlers{listeners: l, playlist: p, songList: sl,
		mostLSong: mls, fallbackSong: fS}
}

func (dj *DJHandlers) ListenersUpdate(w http.ResponseWriter, r *http.Request) {
	defer func() {
		if err := recover(); err != nil {
			log.Println("DJHandlers.ListenersUpdate panic:", err)
		}
	}()

	if err := r.ParseForm(); err != nil {
		log.Println("DJHandlers.ListenersUpdate:", err)
		http.Error(w, "cannot parse form", http.StatusBadRequest)
		return
	}

	switch r.FormValue("action") {
	case "listener_add":
		l := dj.listeners.Inc()
		go dj.songList.UpdateCurrentMaxListeners(l)
	case "listener_remove":
		if _, err := dj.listeners.Dec(); err != nil {
			log.Println("DJHandlers.ListenersUpdate:", err)
			http.Error(w, err.Error(), http.StatusBadRequest)
			return
		}
	default:
		w.WriteHeader(http.StatusNotAcceptable)
		return
	}

	w.Header().Add("Content-Type", "text/plain")
	w.Header().Add("Icecast-Auth-User", "1")
	w.WriteHeader(http.StatusOK)
}

func (dj *DJHandlers) PlaylistNext(w http.ResponseWriter, _ *http.Request) {
	w.Header().Add("Content-Type", "text/plain")
	nxt := dj.playlist.Next()
	if nxt == "" {
		log.Println("the end of a playlist has been reached")
		if nxt = dj.fallbackSong; nxt == "" {
			log.Println("a fallback song is not set")
			http.Error(w, "a playlist is empty and a fallback song is not set", http.StatusNotFound)
			return
		}
	}

	go func() {
		oggf, err := oggtag.NewOggFile(nxt)
		if err != nil {
			log.Println("cannot read an OGG file", nxt, ":", err)
			http.Error(w, "cannot read an OGG file", http.StatusInternalServerError)
			return
		}

		song := radio.Song{
			Artist:       oggf.GetTag("artist"),
			Title:        oggf.GetTag("title"),
			Duration:     oggf.GetDuration(),
			MaxListeners: dj.listeners.Current(),
			// Here 5 seconds are being added because it is approximately the
			// time between the creation of this Song object and when ezstream
			// actually starts to play it.
			StartAt: time.Now().Add(5 * time.Second)}

		if strings.HasSuffix(nxt, "/fallback.ogg") {
			song.Artist = "Nothing to play. Playing a fallback: " + song.Artist
		}

		if dj.songList.Current() != nil {
			dj.mostLSong.Update(*dj.songList.Current())
		}

		dj.songList.Add(song)
	}()
	fmt.Fprintln(w, nxt)
}

func (dj *DJHandlers) Status(w http.ResponseWriter, r *http.Request) {
	w.Header().Add("Content-Type", "application/json")
	err := json.NewEncoder(w).Encode(&struct {
		Current   *radio.Song             `json:"current_song,omitempty"`
		Listeners *radio.ListenerCounter  `json:"listeners"`
		List      []radio.Song            `json:"last_songs,omitempty"`
		Mls       *radio.MostListenedSong `json:"most_listened_song,omitempty"`
	}{
		Current:   dj.songList.Current(),
		Listeners: dj.listeners,
		List:      dj.songList.List(),
		Mls:       dj.mostLSong.Get()})
	if err != nil {
		log.Println("DJHandlers.Status:", err)
		http.Error(w, "status parsing failed", http.StatusInternalServerError)
	}
}
